#!/bin/bash -e
export IMAGE_TYPE="$1"

if [[ -z "$OS_VERSION" ]]; then
    export OS_VERSION="$2"
fi

source "$OS_FAMILY"Dep.sh

# Global version file
VERSION_FILE="version.json"

# Initialize the global version file
echo "{" > "$VERSION_FILE"

echo "Recieved input: Common Deps - $COMMON_DEPS and extra Deps - $EXTRA_DEPS and patches $DEP_PATCHES"
IFS=" " read -ra common_functions <<< "$COMMON_DEPS"
IFS=" " read -ra extra_functions <<< "$EXTRA_DEPS"
IFS=" " read -ra dep_patches <<< "$DEP_PATCHES"


for dep in "${common_functions[@]}"; do
    func="${dep%%:*}"
    version="${dep#*:}"

    for extra_dep in "${extra_functions[@]}"; do
        extra_func="${extra_dep%%:*}"
        extra_func_version="${extra_dep#*:}"

        if [[ "$func" == "$extra_func" ]];then
            echo "Version is different in the extra dep - updating version to $extra_func_version for $func"
            version="$extra_func_version"
        fi
    done;

    for patch in "${dep_patches[@]}"; do
        patch_func="${patch%%:*}"
        patch_version="${patch#*:}"

        if [[ "$func" == "$patch_func" ]];then
            echo "Patching version $version for $func"
            version="$patch_version"
        fi
    done;
    
    function_name="install_$func"

    if [ "$(type -t "$function_name")" = "function" ]; then
        echo "Added function $func for execution to Packer - with version $version"
        "$function_name" "$version"
    fi

done;

for dep in "${extra_functions[@]}"; do
    func="${dep%%:*}"
    version="${dep#*:}"

    for common_dep in "${common_functions[@]}"; do
        common_func="${common_dep%%:*}"
        common_func_version="${common_dep#*:}"

        if [[ "$func" == "$common_func" ]];then
            echo "Dep is already installed - skipping"
            continue 2
        fi
    done;

    for patch in "${dep_patches[@]}"; do
        patch_func="${patch%%:*}"
        patch_version="${patch#*:}"

        if [[ "$func" == "$patch_func" ]];then
            echo "Patching version $version for $func"
            version="$patch_version"
        fi
    done;
    
    function_name="install_$func"

    if [ "$(type -t "$function_name")" = "function" ]; then
        echo "Added function $func for execution to Packer - with version $version"
        "$function_name" "$version"
    fi

done;


# Remove the trailing comma from the version file
sed -i '$ s/,$//' "$VERSION_FILE"

# Add the closing curly brace
echo "}" >> "$VERSION_FILE"

cat $VERSION_FILE